@@ -48,6 +48,7 @@
 
 #include <errno.h>
 #include <syslog.h>
+#include <libgen.h>
 
 #include <guacamole/client.h>
 #include <guacamole/error.h>
@@ -82,7 +83,7 @@ void guacd_handle_connection(int fd) {
     if (select->argc != 1) {
 
         /* Log error */
-        syslog(LOG_ERR, "Bad number of arguments to \"select\" (%i)",
+        guacd_log_error("Bad number of arguments to \"select\" (%i)",
                 select->argc);
 
         /* Free resources */
@@ -90,7 +91,7 @@ void guacd_handle_connection(int fd) {
         return;
     }
 
-    syslog(LOG_INFO, "Protocol \"%s\" selected", select->argv[0]);
+    guacd_log_info("Protocol \"%s\" selected", select->argv[0]);
 
     /* Get plugin from protocol in select */
     plugin = guac_client_plugin_open(select->argv[0]);
@@ -137,7 +138,9 @@ void guacd_handle_connection(int fd) {
 
     /* Load and init client */
     client = guac_client_plugin_get_client(plugin, socket,
-            connect->argc, connect->argv); 
+            connect->argc, connect->argv,
+            guacd_client_log_info, guacd_client_log_error);
+
     guac_instruction_free(connect);
 
     if (client == NULL) {
@@ -151,21 +154,17 @@ void guacd_handle_connection(int fd) {
         return;
     }
 
-    /* Set up logging in client */
-    client->log_info_handler  = guacd_log_info;
-    client->log_error_handler = guacd_log_error;
-
     /* Start client threads */
-    syslog(LOG_INFO, "Starting client");
+    guacd_log_info("Starting client");
     if (guacd_client_start(client))
-        syslog(LOG_ERR, "Client finished abnormally");
+        guacd_log_error("Client finished abnormally");
     else
-        syslog(LOG_INFO, "Client finished normally");
+        guacd_log_info("Client finished normally");
 
     /* Clean up */
     guac_client_free(client);
     if (guac_client_plugin_close(plugin))
-        syslog(LOG_ERR, "Error closing client plugin");
+        guacd_log_error("Error closing client plugin");
 
     /* Close socket */
     guac_socket_close(socket);
@@ -184,6 +183,12 @@ int main(int argc, char* argv[]) {
     char bound_port[64];
     int opt_on = 1;
 
+    struct addrinfo hints = {
+        .ai_family   = AF_UNSPEC,
+        .ai_socktype = SOCK_STREAM,
+        .ai_protocol = IPPROTO_TCP
+    };
+
     /* Client */
     struct sockaddr_in client_addr;
     socklen_t client_addr_len;
@@ -223,9 +228,13 @@ int main(int argc, char* argv[]) {
         }
     }
 
+    /* Set up logging prefix */
+    strncpy(log_prefix, basename(argv[0]), sizeof(log_prefix));
+
+
     /* Get addresses for binding */
-    if ((retval = getaddrinfo(listen_address, listen_port, NULL, &addresses))) {
-        fprintf(stderr, "Error parsing given address or port: %s\n",
+    if ((retval = getaddrinfo(listen_address, listen_port, &hints, &addresses))) {
+        guacd_log_error("Error parsing given address or port: %s",
                 gai_strerror(retval));
         exit(EXIT_FAILURE);
     }
@@ -233,32 +242,48 @@ int main(int argc, char* argv[]) {
     /* Get socket */
     socket_fd = socket(AF_INET, SOCK_STREAM, 0);
     if (socket_fd < 0) {
-        fprintf(stderr, "Error opening socket: %s\n", strerror(errno));
+        guacd_log_error("Error opening socket: %s", strerror(errno));
         exit(EXIT_FAILURE);
     }
 
     /* Allow socket reuse */
     if (setsockopt(socket_fd, SOL_SOCKET, SO_REUSEADDR, (void*) &opt_on, sizeof(opt_on))) {
-        fprintf(stderr, "Warning: Unable to set socket options for reuse: %s\n", strerror(errno));
+        guacd_log_info("Unable to set socket options for reuse: %s", strerror(errno));
     }
 
     /* Attempt binding of each address until success */
     current_address = addresses;
     while (current_address != NULL) {
 
+        int retval;
+
+        /* Resolve hostname */
+        if ((retval = getnameinfo(current_address->ai_addr,
+                current_address->ai_addrlen,
+                bound_address, sizeof(bound_address),
+                bound_port, sizeof(bound_port),
+                NI_NUMERICHOST | NI_NUMERICSERV)))
+            guacd_log_error("Unable to resolve host: %s",
+                    gai_strerror(retval));
+
         /* Attempt to bind socket to address */
         if (bind(socket_fd,
                     current_address->ai_addr,
                     current_address->ai_addrlen) == 0) {
 
+            guacd_log_info("Successfully bound socket to "
+                    "host %s, port %s", bound_address, bound_port);
+
             /* Done if successful bind */
             break;
 
         }
 
-        /* Otherwise log error */
+        /* Otherwise log information regarding bind failure */
         else
-            fprintf(stderr, "Error binding socket: %s\n", strerror(errno));
+            guacd_log_info("Unable to bind socket to "
+                    "host %s, port %s: %s",
+                    bound_address, bound_port, strerror(errno));
 
         current_address = current_address->ai_next;
 
@@ -266,7 +291,7 @@ int main(int argc, char* argv[]) {
 
     /* If unable to bind to anything, fail */
     if (current_address == NULL) {
-        fprintf(stderr, "Unable to bind socket to any addresses.\n");
+        guacd_log_error("Unable to bind socket to any addresses.");
         exit(EXIT_FAILURE);
     }
 
@@ -275,7 +300,7 @@ int main(int argc, char* argv[]) {
 
     /* If error, fail */
     if (daemon_pid == -1) {
-        fprintf(stderr, "Error forking daemon process: %s\n", strerror(errno));
+        guacd_log_error("Error forking daemon process: %s", strerror(errno));
         exit(EXIT_FAILURE);
     }
 
@@ -293,7 +318,7 @@ int main(int argc, char* argv[]) {
 
             /* Warn on failure */
             else {
-                fprintf(stderr, "WARNING: Could not write PID file: %s\n", strerror(errno));
+                guacd_log_error("Could not write PID file: %s", strerror(errno));
                 exit(EXIT_FAILURE);
             }
 
@@ -307,23 +332,17 @@ int main(int argc, char* argv[]) {
 
     /* Ignore SIGPIPE */
     if (signal(SIGPIPE, SIG_IGN) == SIG_ERR) {
-        syslog(LOG_ERR, "Could not set handler for SIGPIPE to ignore. SIGPIPE may cause termination of the daemon.");
+        guacd_log_info("Could not set handler for SIGPIPE to ignore. SIGPIPE may cause termination of the daemon.");
     }
 
     /* Ignore SIGCHLD (force automatic removal of children) */
     if (signal(SIGCHLD, SIG_IGN) == SIG_ERR) {
-        syslog(LOG_ERR, "Could not set handler for SIGCHLD to ignore. Child processes may pile up in the process table.");
+        guacd_log_info("Could not set handler for SIGCHLD to ignore. Child processes may pile up in the process table.");
     }
 
-    /* Log address and port */
-    if (getnameinfo(current_address->ai_addr, current_address->ai_addrlen,
-            bound_address, sizeof(bound_address),
-            bound_port, sizeof(bound_port),
-            0))
-        syslog(LOG_WARNING, "Could not resolve name of listening host.");
-    else
-        syslog(LOG_INFO,
-                "Listening on host %s, port %s", bound_address, bound_port);
+    /* Log listening status */
+    syslog(LOG_INFO,
+            "Listening on host %s, port %s", bound_address, bound_port);
 
     /* Free addresses */
     freeaddrinfo(addresses);
@@ -335,7 +354,7 @@ int main(int argc, char* argv[]) {
 
         /* Listen for connections */
         if (listen(socket_fd, 5) < 0) {
-            syslog(LOG_ERR, "Could not listen on socket: %s", strerror(errno));
+            guacd_log_error("Could not listen on socket: %s", strerror(errno));
             return 3;
         }
 
@@ -343,7 +362,7 @@ int main(int argc, char* argv[]) {
         client_addr_len = sizeof(client_addr);
         connected_socket_fd = accept(socket_fd, (struct sockaddr*) &client_addr, &client_addr_len);
         if (connected_socket_fd < 0) {
-            syslog(LOG_ERR, "Could not accept client connection: %s", strerror(errno));
+            guacd_log_error("Could not accept client connection: %s", strerror(errno));
             return 3;
         }
 
@@ -360,7 +379,7 @@ int main(int argc, char* argv[]) {
 
         /* If error, log */
         if (child_pid == -1)
-            syslog(LOG_ERR, "Error forking child process: %s\n", strerror(errno));
+            guacd_log_error("Error forking child process: %s", strerror(errno));
 
         /* If child, start client, and exit when finished */
         else if (child_pid == 0) {
@@ -371,14 +390,14 @@ int main(int argc, char* argv[]) {
 
         /* If parent, close reference to child's descriptor */
         else if (close(connected_socket_fd) < 0) {
-            syslog(LOG_ERR, "Error closing daemon reference to child descriptor: %s", strerror(errno));
+            guacd_log_error("Error closing daemon reference to child descriptor: %s", strerror(errno));
         }
 
     }
 
     /* Close socket */
     if (close(socket_fd) < 0) {
-        syslog(LOG_ERR, "Could not close socket: %s", strerror(errno));
+        guacd_log_error("Could not close socket: %s", strerror(errno));
         return 3;
     }
 
